import turtle

#print(turtle.window_width(), turtle.window_height())


def main():
    scr = initializescreen()
    t1 = initializeturtle()

    heptagonpattern(t1, 0)
    threesegmentcircle(t1, 3)
    sixsectorcircle(t1, 6)
    flower(t1, 0)
    turtlestamp(t1, 1)
    circularspiral(t1, 0)
    stars(t1, 4)
    rectangles(t1, 1)

    scr.exitonclick()


def initializescreen():
    screen = turtle.Screen()
    screen.setup(width=1000, height=850, startx=None, starty=None)
    screen.title('Turtle Graphics')
    screen.colormode(255)
    screen.bgcolor('white')
    screen.mode('logo')  # Logo mode sets Up as 0°, Right as 90°, Down as 180°, Left as 270°, etc.
    return screen


def initializeturtle(shape='turtle', pen='black', fill='white'):
    t = turtle.Turtle()  # Create t as a turtle instance.
    t.shape(shape)       # Draws the turtle as a set shape.
    t.speed(0)           # Sets drawing speed to maximum - others are 1 (slow) - 10 (fast).
    t.clear()            # Clears the screen.
    t.home()             # Puts the turtle at the screen centre pointing Up.
    t.pu()               # Sets pen up (i.e. not drawing).
    t.pencolor(pen)      # Sets the pen colour.
    t.fillcolor(fill)    # Sets the fill colour.
    t.pensize(1)         # Sets the pen width to 1 (minimum).
    return t


def heptagonpattern(t, speed=0):
    t.home()
    t.lt(90-360/14)
    t.speed(speed)
    if speed == 0:
        t.ht()
    t.pensize(8)
    t.fillcolor('yellow')
    t.begin_fill()
    for i in range(12):
        polygon(t, 7, 150, 'red')
        t.rt(30)
    t.end_fill()
    t.st()
    t.home()
    t.pu()
    t.bk(380)
    write(t, 'Heptagon pattern', 18, 'purple')
    t.fd(380)
    t.pensize(1)
    t.speed(0)


def write(t, word, size, colour='black'):
    t.pencolor(colour)
    t.write(word, False, 'center', ('Arial', size, 'normal'))    


def polygon(t, sides, size, colour='black'):
    if sides < 3 or size < 1:
        return
    t.pencolor(colour)
    t.pd()
    for i in range(sides):
        t.fd(size)
        t.rt(360/sides)
    t.pu()


def square(t, size, colour='black'):
    t.pencolor(colour)
    t.pd()
    polygon(t, 4, size, colour)
    t.pu()


def threesegmentcircle(t, speed=0):
    t.pu()
    t.setpos(270,-270)
    t.pd()
    t.speed(speed)
    t.pensize(2)
    t.fillcolor('pink')
    for i in range(3):
        t.begin_fill()
        segment(t, 100, 120, 'blue', 'R')
        t.end_fill()
    t.pu()
    t.rt(90)
    t.fd(100)
    t.pd()
    t.dot()
    t.pu()
    t.bk(100)
    t.lt(90)
    t.pensize(1)
    t.speed(0)


def segment(t, radius, angle, colour='black', lr='R'):
    if radius < 1 or angle < 0 or angle > 360 or lr not in ['L','R']:
        return
    if lr == 'R':
        radius = -radius
    t.pencolor(colour)
    t.pd()
    t.circle(radius, angle)
    t.pu()


def sixsectorcircle(t, speed=0):
    t.pu()
    t.setpos(-370,-270)
    t.pd()
    t.speed(speed)
    t.pensize(2)
    fillcolours = ['red','green','blue','yellow','magenta','cyan']
    for i in range(6):
        t.fillcolor(fillcolours[i])
        t.begin_fill()
        sector(t, 100, 60, 'black', 'L')
        t.end_fill()
        t.lt(60)
    t.pu()
    t.pensize(1)
    t.speed(0)


def sector(t, radius, angle, colour='black', lr='R'):
    if radius < 1 or angle < 0 or angle > 360 or lr not in ['L','R']:
        return
    angle = angle % 360
    if angle == 0:
        angle = 360
    x = 1
    if lr == 'L':
        x = -1
    t.pencolor(colour)
    t.pd()
    if angle == 360:
        t.pu()
    t.fd(radius)
    t.pd()
    t.rt(90*x)
    t.circle(-radius*x, angle)
    t.rt(90*x)
    if angle == 360:
        t.pu()
    t.fd(radius)
    t.rt((180-angle)*x)
    t.pu()


def flower(t, speed=0):
    t.speed(speed)
    t.pu()
    t.setpos(-375,300)
    t.ht()
    for i in range(24):
        t.pu()
        t.fd(40)
        t.pd()
        t.fillcolor('orange')
        t.begin_fill()
        vesica(t, 40, 90, 'orange')
        t.end_fill()
        t.pu()
        t.bk(40)
        t.rt(15)
    t.pd()
    t.fillcolor('yellow')
    t.begin_fill()
    sector(t, 50, 360, 'yellow', 'R')
    t.end_fill()
    t.pu()
    t.home()
    t.pencolor('black')
    t.st()


def vesica(t, size, angle, colour='black'):
    t.lt(angle/2)
    if size < 10 or angle < 30 or angle > 150:
        return
    segment(t, size, angle, colour, 'R')
    t.rt(180-angle)
    segment(t, size, angle, colour, 'R')
    t.rt(180-angle/2)


def circularspiral(t, speed=0):
    t.pu()
    t.setpos(365,300)
    t.speed(speed)
    if speed == 0:
        t.ht()
    t.pensize(1)
    t.pencolor('purple')
    t.pd()
    spiral(t, 20, 25, 1.5, 'R')
    t.st()
    t.pu()
    t.speed(0)


def spiral(t, sides, size, gap=1, lr='R'):
    if sides < 3 or sides > 20 or gap < 1 or lr not in ['L','R']:
        return
    x = gap
    d = 1
    if lr == 'L':
        d = -1
    for i in range(size*sides):
        t.fd(x)
        t.rt(d*360/sides)
        x = x+gap*(2/(sides-2))**1.5


def turtlestamp(t, speed=0):
    t.home()
    t.pu()
    t.bk(230)
    t.lt(90-360/14)
    t.speed(speed)
    t.pencolor('black')
    fillcolours = ['red', 'orange', 'yellow', 'green', 'cyan', 'blue', 'purple']
    for i in range(7):
        t.fillcolor(fillcolours[i])
        t.shapesize(i * 0.5 + 1, i * 0.5 + 1, 1)
        t.fd(200)
        t.rt(360/7)
        t.stamp()
    t.speed(0)
    t.shapesize(1.0, 1.0, 1)
    t.seth(0)
    t.home()


def stars(t, speed=0):
    t.speed(speed)
    t.pu()
    t.pensize(2)
    t.pencolor('blue')
    t.fillcolor('green')
    t.setpos(-450, -60)
    t.seth(0)
    star(t, 7, 130, True)
    t.pencolor('purple')
    t.fillcolor('magenta')   # Unnecessary if star(,,,filled) is False
    t.setpos(375, -55)
    t.seth(0)
    star(t, 5, 130)


def star(t, points, size, filled=False):
    if points < 4:
        return
    x = 1
    if points % 2 == 0:
        x = 2
    t.pd()
    t.rt(90/points)
    if filled == True:
        t.begin_fill()
    for i in range(points*x):
        t.fd(size)
        t.rt(180-180/points)
    if filled == True:
        t.end_fill()
    t.pu()


def rectangles(t, speed=0):
    t.pu()
    t.pensize(4)
    t.setpos(-75, 340)
    t.seth(0)
    t.speed(speed)
    t.pencolor('dark blue')
    rectangle(t, 70, 150)
    t.speed(0)
    t.setpos(-250, 300)
    t.speed(speed)
    t.seth(0)
    t.pencolor('brown')
    t.fillcolor('dark cyan')
    t.begin_fill()
    rectangle(t, 100, 25)
    t.end_fill()
    t.speed(0)
    t.setpos(225, 300)
    t.speed(speed)
    t.pencolor('olive drab')
    t.fillcolor('light green')
    t.begin_fill()
    rectangle(t, 100, 25)
    t.end_fill()
    t.speed(0)


def rectangle(t, side1, side2):
    if side1 < 1 or side2 < 1:
        return
    t.pd()
    for i in range(2):
        t.fd(side1)
        t.rt(90)
        t.fd(side2)
        t.rt(90)
    t.pu()


if __name__ == '__main__':
    main()


'''
Main Python Turtle Commands

t.clear()       Clear screen
t.home()        Put turtle at screen centre pointing up
t.speed()       Set speed of turtle movement - has one parameter (0-10)
t.st()          Show turtle
t.ht()          Hide turtle (drawing is faster with turtle hidden)
t.setpos()      Set turtle position - has 2 parameters for x and y position
t.seth()        Set turtle heading - has 1 parameter (0-360)
t.fd()          Move forwards - has 1 parameter for distance
t.bk()          Move backward - has 1 parameter for distance
t.rt()          Turn right - has 1 parameter for number of degrees
t.lt()          Turn left  - has 1 parameter for number of degrees
t.circle()      Draw an arc - has 2 main parameters for radius and angle turned
t.pd()          Pen down
t.pu()          Pen up
t.pensize()     Pen width   - has 1 parameter for width
t.pencolor()    Pen colour  - has 1 constant or 3 RGB values as parameters
t.fillcolor()   Fill colour - has 1 constant or 3 RGB values as parameters
t.begin_fill()  Shapes drawn between this and t.end_fill are filled in
t.end_fill()    Ends filling in started by t.begin_fill
t.dot()         Put a dot at the current position
t.stamp()       Put a turtle at the current position and heading
t.write()       Writes text - details of text, alignment, font, etc. as parameters


Useful Colour Names

black, white, gray, light gray
red, dark red, maroon, salmon, pink
orange, dark orange, coral
yellow, gold, goldenrod
green, dark green, light green, olive drab
cyan, dark cyan, turquoise, aquamarine, sky blue
blue, dark blue, cornflower blue
purple, violet, magenta, orchid, slate blue
brown, sienna, chocolate, khaki, beige, tan

'''
