import turtle
import TurtleUtilsPMP as tp


def main():
    scr = tp.initializescreen()
    # 1000, 850, 'white', 'Turtle Graphics' are defaults for width, height, colour, and title
    ada = tp.initializeturtle()
    # 'turtle', 'black', 'white' are defaults for shape, colour, and fill

    #scr.tracer(100)
    # Accelerates drawing by updating screen only every n frames.
    tp.gridxy(ada)
    
    heptagonpattern(ada)
    threesegmentcircle(ada, 3)
    sixsectorcircle(ada, 6)
    flower(ada)
    turtlestamp(ada, 1)
    circularspiral(ada)
    stars(ada, 4)
    ellipses(ada, 0)
    crescents(ada, 4)
    rectangles(ada, 1)
    ovals(ada, 1)

    scr.exitonclick()


def heptagonpattern(t, speed=0):
    t.home()
    t.lt(90-360/14)
    t.speed(speed)
    if speed == 0:
        t.ht()
    t.pensize(8)
    t.fillcolor('yellow')
    t.begin_fill()
    # Needed to make the whole shape filled rather than each component heptagon
    for i in range(12):
        tp.polygon(t, 7, 150, 'red')
        # 'white', False, 'R' are default values for fill, filled, and direction
        t.rt(30)
    t.end_fill()
    t.st()
    t.home()
    t.pu()
    t.bk(400)
    tp.write(t, 'PMP Python Turtle Graphics Utilities\nDemo', 18, 'purple', face='bold')
    # 'center', 'Arial', 'normal', False are defaults for alignment, font, type, and offset
    t.fd(380)
    t.pensize(1)
    t.speed(0)


def threesegmentcircle(t, speed=0):
    t.pu()
    t.setpos(270,-270)
    t.speed(speed)
    t.pensize(2)
    for i in range(3):
        tp.segment(t, 100, 120, 'blue', 'pink', True)
        # 'R' is default for direction
    t.rt(90)
    t.fd(100)
    t.pd()
    t.dot()
    t.pu()
    t.bk(100)
    t.lt(90)
    t.pensize(1)
    t.speed(0)


def sixsectorcircle(t, speed=0):
    t.pu()
    t.setpos(-370,-270)
    t.speed(speed)
    t.pensize(2)
    fillcolours = ['red', 'green', 'blue', 'yellow', 'magenta', 'cyan']
    for i in range(6):
        tp.sector(t, 100, 60, fill=fillcolours[i], filled=True, lr='L')
        # 'black', True are defaults for pen colour and centred
        t.lt(60)
    t.pensize(1)
    t.speed(0)


def circularspiral(t, speed=0):
    t.pu()
    t.setpos(365,300)
    t.speed(speed)
    if speed == 0:
        t.ht()
    t.pensize(1)
    tp.spiral(t, 20, 25, 1.5, 'purple')
    # 'R' is default for direction
    t.st()
    t.speed(0)


def turtlestamp(t, speed=0):
    t.home()
    t.pu()
    t.bk(230)
    t.lt(90-360/14)
    t.speed(speed)
    t.pencolor('black')
    fillcolours = ['red', 'orange', 'yellow', 'green', 'cyan', 'blue', 'purple']
    for i in range(7):
        t.fillcolor(fillcolours[i])
        t.shapesize(i * 0.5 + 1, i * 0.5 + 1, 1)
        t.fd(200)
        t.rt(360/7)
        t.stamp()
    t.speed(0)
    t.shapesize(1.0, 1.0, 1)
    t.seth(0)
    t.home()


def flower(t, speed=0):
    t.speed(speed)
    t.pu()
    t.setpos(-375,300)
    t.ht()
    for i in range(24):
        t.pu()
        t.fd(40)
        tp.vesica(t, 40, 90, 'orange', 'orange', True)
        t.bk(40)
        t.rt(15)
    tp.circle(t, 50, 'yellow', 'yellow', True, True)
    t.home()
    t.pencolor('black')
    t.st()
    t.speed(0)


def stars(t, speed=0):
    t.pu()
    t.pensize(2)
    t.setpos(-450, -60)
    t.seth(0)
    t.speed(speed)
    tp.star(t, 7, 130, 'blue', 'green', True)
    t.speed(0)
    t.setpos(375, -55)
    t.seth(0)
    t.speed(speed)
    tp.star(t, 5, 130, 'purple')
    # False is default for filled
    t.speed(0)


def ellipses(t, speed=0):
# If the fourth parameter of tp.ellipse is between 4 and 12, a flattened polygon is drawn.
    t.pu()
    t.pensize(4)
    t.setpos(-465, 150)
    t.seth(0)
    t.rt(30)
    t.speed(speed)
    tp.ellipse(t, 50, 25, 20, 'turquoise', lr='R')
    t.speed(0)
    t.setpos(465, 150)
    t.seth(0)
    t.lt(30)
    t.speed(speed)
    tp.ellipse(t, 50, 25, 20, 'coral', 'slate blue', True, lr='L')
    t.speed(0)


def crescents(t, speed=0):
    t.pu()
    t.pensize(2)
    t.setpos(-400, -150)
    t.seth(0)
    t.speed(speed)
    tp.crescent(t, 50, 90, 'maroon', 'coral', True, 'L')
    t.speed(0)
    t.setpos(400, -150)
    t.seth(0)
    t.speed(speed)
    tp.crescent(t, 50, 90, 'sienna', 'orchid', True, 'R')
    t.speed(0)


def rectangles(t, speed=0):
    t.pu()
    t.pensize(4)
    t.setpos(0, 375)
    t.seth(0)
    t.speed(speed)
    tp.rectangle(t, 70, 150, 'dark blue', centred=True)
    tp.write(t, 'Rectangle', 18, 'dark orange', 'center', 'Arial', 'bold', True)
    # Last parameter as True gives vertical centring.
    tp.write(t, 'Centred\nsmall\nnew\nblue\nrectangle', 8, 'blue', 'center', 'Arial', 'italic', True)
    t.speed(0)
    t.setpos(-250, 300)
    t.speed(speed)
    t.seth(0)
    tp.rectangle(t, 100, 25, 'brown', 'dark cyan', True)
    t.speed(0)
    t.setpos(225, 300)
    t.speed(speed)
    tp.rectangle(t, 100, 25, 'olive drab', 'light green', True)
    t.speed(0)


def ovals(t, speed=0):
    t.pu()
    t.pensize(4)
    t.setpos(-185, 345)
    t.seth(0)
    t.rt(45)
    t.speed(speed)
    tp.oval(t, 65, 35, 'dark green', 'gold', True)
    t.speed(0)
    t.setpos(185, 345)
    t.seth(0)
    t.lt(45)
    t.speed(speed)
    tp.oval(t, 65, 35, 'dark red', 'goldenrod', True, lr='L')
    # An alternative is  tp.rectangle(t, 65, 35, 'dark red', 'goldenrod', True, lr = 'L', oval=True)
    t.speed(0)


if __name__ == '__main__':
    main()


'''
Main Python Turtle Commands

t.clear()       Clear screen
t.home()        Put turtle at screen centre pointing up
t.speed()       Set speed of turtle movement - has one parameter (0-10)
t.st()          Show turtle
t.ht()          Hide turtle (drawing is faster with turtle hidden)
t.setpos()      Set turtle position - has 2 parameters for x and y position
t.seth()        Set turtle heading - has 1 parameter (0-360)
t.fd()          Move forwards - has 1 parameter for distance
t.bk()          Move backward - has 1 parameter for distance
t.rt()          Turn right - has 1 parameter for number of degrees
t.lt()          Turn left  - has 1 parameter for number of degrees
t.circle()      Draw an arc - has 2 main parameters for radius and angle turned
t.pd()          Pen down
t.pu()          Pen up
t.pensize()     Pen width   - has 1 parameter for width
t.pencolor()    Pen colour  - has 1 constant or 3 RGB values as parameters
t.fillcolor()   Fill colour - has 1 constant or 3 RGB values as parameters
t.begin_fill()  Shapes drawn between this and t.end_fill are filled in
t.end_fill()    Ends filling in started by t.begin_fill
t.dot()         Put a dot at the current position
t.stamp()       Put a turtle at the current position and heading
t.write()       Writes text - details of text, alignment, font, etc. as parameters


Useful Colour Names

black, white, gray, light gray
red, dark red, maroon, salmon, pink
orange, dark orange, coral
yellow, gold, goldenrod
green, dark green, light green, olive drab
cyan, dark cyan, turquoise, aquamarine, sky blue
blue, dark blue, cornflower blue
purple, violet, magenta, orchid, slate blue
brown, sienna, chocolate, khaki, beige, tan


PMP Turtle Utils Commands

initializescreen(width=1000, height=850, colour='white', title='Turtle Graphics')
initializeturtle(shape='turtle', pen='black', fill='white')
write(turtle, text, fontsize=14, colour='black', align='center', font='Arial', face='normal')
gridxy(t, spacing=100, gridcolour='light gray', numbercolour='gray', numbered=True,
    startx=-500, starty=-500)
polygon(turtle, sides, size, pen='black', fill='white', filled=False, centred=False, lr='R')
square(turtle, size, pen='black', fill='white', filled=False, centred=False, lr='R')
rectangle(turtle, width, height, pen='black', fill='white', filled=False, centred=False,
    lr='R', oval=False)
oval(turtle, width, height, pen='black', fill='white', filled=False, centred=False, lr='R')
segment(turtle, radius, angle, pen='black', fill='white', filled=False, lr='R')
sector(turtle, radius, angle, pen='black', fill='white', filled=False, centred=True, lr='R')
circle(turtle, radius, pen='black', fill='white', filled=False, centred=True, lr='R')
ellipse(turtle, long radius, short radius, sides=20, pen='black', fill='white',
    filled=False, centred=False, lr='R', semi=False)
vesica(turtle, size [>=10], angle [30-159], pen='black', fill='white', filled=False) 
spiral(turtle, sides=4, size=100, gap=1, pen='black', lr='R')
star(turtle, points, size, pen='black', fill='white', filled=False, lr='R')
crescent(turtle, size [>=10], angle [30-120], pen='black', fill='white', filled=False, lr='R')

'''
