import turtle


def main():
    scr = initializescreen()  # This sets up the turtle graphics drawing canvas (with the name 'scr').
    Ada = initializeturtle()  # This creates a turtle called Ada.
    setup(scr, Ada)           # This puts Ada on a canvas with a maze background.


    '''
    The aim is to work out commands to move Ada through the maze without touching a wall.
    Enter your commands to move Ada below this line.
    (The first two sections of the route are given to get you started.)
    '''
    Ada.fd(92)   # This moves Ada forward 92 units (pixels).
    Ada.lt(90)   # This turns Ada 90° left.
    Ada.fd(170)  # This moves Ada forward 170 units.
    Ada.rt(90)   # This turns Ada 90° right.


    '''
    Finish your commands to move the turtle through the maze above this line.
    '''
    
    scr.exitonclick()  # This closes the canvas window when you click on it.


def setup(s, t):
    s.bgpic('Maze 01 1000x850.gif')
    # This sets a maze as the canvas background picture.
    t.turtlesize(1.5, 1.5, 2)
    # This sets the turtle's width and length to 1.5x normal and outline thickness to 2x normal.
    t.pu()
    t.setpos(0, 350)
    t.pd()
    t.pencolor('red')
    t.write('MAZE', False, 'center', ('Arial', 36, 'bold'))
    t.pu()
    t.setpos(0, -385)
    t.pd()
    t.pencolor('green')
    t.write('Draw a line through the maze without touching any blue wall.', \
        False, 'center', ('Arial', 18, 'bold'))
    t.pu()
    t.setpos(-480, -85)  # This sets the turtle starting position.
    t.seth(90)           # This sets the turtle starting heading.
    t.pd()
    t.pencolor('red')
    t.pensize(8)
    t.speed(1)           # This sets the drawing speed to its minimum.


def initializescreen():
    screen = turtle.Screen()  # This sets up a canvas (i.e. screen) for the turtle to draw on.
    screen.setup(width=1000, height=850, startx=None, starty=None)  # This sets the canvas size.
    screen.title('Turtle Graphics')  # This puts a title in the title bar of the canvas.
    screen.colormode(255)     # This allows use of normal rgb colours.
    screen.bgcolor('white')   # This sets the background colour of the canvas to white.
    screen.mode('logo')  # Logo mode sets Up as 0°, Right as 90°, Down as 180°, Left as 270°, etc.
    return screen


def initializeturtle(shape='turtle', pen='black', fill='white'):
    t = turtle.Turtle()  # This creates t as a turtle instance.
    t.shape(shape)       # This draws the turtle as a set shape (a turtle by default).
    t.speed(0)           # This sets drawing speed to maximum - others are 1 (slow) - 10 (fast).
    t.clear()            # This clears the canvas (i.e. screen).
    t.home()             # This puts the turtle at the canvas centre pointing Up.
    t.pu()               # This sets pen up (so that the turtle moves without drawing).
    t.pencolor(pen)      # This sets the pen colour (black by default).
    t.fillcolor(fill)    # This sets the fill colour (white by default).
    t.pensize(1)         # This sets the pen width to 1 (minimum).
    return t


if __name__ == '__main__':
    main()


'''
Main Python Turtle Commands

t.clear()       Clear screen
t.home()        Put turtle at screen centre pointing up
t.speed()       Set speed of turtle movement - has one parameter (0-10)
t.st()          Show turtle
t.ht()          Hide turtle (drawing is faster with turtle hidden)
t.setpos()      Set turtle position - has 2 parameters for x and y position
t.seth()        Set turtle heading - has 1 parameter (0-360)
t.fd()          Move forwards - has 1 parameter for distance
t.bk()          Move backward - has 1 parameter for distance
t.rt()          Turn right - has 1 parameter for number of degrees
t.lt()          Turn left  - has 1 parameter for number of degrees
t.circle()      Draw an arc - has 2 main parameters for radius and angle turned
t.pd()          Pen down
t.pu()          Pen up
t.pensize()     Pen width   - has 1 parameter for width
t.pencolor()    Pen colour  - has 1 constant or 3 RGB values as parameters
t.fillcolor()   Fill colour - has 1 constant or 3 RGB values as parameters
t.begin_fill()  Shapes drawn between this and t.end_fill are filled in
t.end_fill()    Ends filling in started by t.begin_fill
t.dot()         Put a dot at the current position
t.stamp()       Put a turtle at the current position and heading
t.write()       Writes text - details of text, alignment, font, etc. as parameters


Useful Colour Names

black, white, gray, light gray
red, dark red, maroon, salmon, pink
orange, dark orange, coral
yellow, gold, goldenrod
green, dark green, light green, olive drab
cyan, dark cyan, turquoise, aquamarine, sky blue
blue, dark blue, cornflower blue
purple, violet, magenta, orchid, slate blue
brown, sienna, chocolate, khaki, beige, tan


PMP Turtle Utils Commands

initializescreen(width=1000, height=850, colour='white', title='Turtle Graphics')
initializeturtle(shape='turtle', pen='black', fill='white')
write(turtle, text, fontsize=14, colour='black', align='center', font='Arial', face='normal')
gridxy(t, spacing=100, gridcolour='light gray', numbercolour='gray', numbered=True,
    startx=-500, starty=-500)
polygon(turtle, sides, size, pen='black', fill='white', filled=False, centred=False, lr='R')
square(turtle, size, pen='black', fill='white', filled=False, centred=False, lr='R')
rectangle(turtle, width, height, pen='black', fill='white', filled=False, centred=False,
    lr='R', oval=False)
oval(turtle, width, height, pen='black', fill='white', filled=False, centred=False, lr='R')
segment(turtle, radius, angle, pen='black', fill='white', filled=False, lr='R')
sector(turtle, radius, angle, pen='black', fill='white', filled=False, centred=True, lr='R')
circle(turtle, radius, pen='black', fill='white', filled=False, centred=True, lr='R')
ellipse(turtle, long radius, short radius, sides=20, pen='black', fill='white',
    filled=False, centred=False, lr='R', semi=False)
vesica(turtle, size [>=10], angle [30-159], pen='black', fill='white', filled=False) 
spiral(turtle, sides=4, size=100, gap=1, pen='black', lr='R')
star(turtle, points, size, pen='black', fill='white', filled=False, lr='R')
crescent(turtle, size [>=10], angle [30-120], pen='black', fill='white', filled=False, lr='R')

'''
