import turtle
import TurtleUtilsPMP as tp
import TurtleCurvesPMP as tc
import tkinter.messagebox as mb


'''

This program draws trochoids; these are produced mechanically (as in a Spirograph set) by rolling
  a small circle inside or outside the circumference of a larger circle with the pen offset from
  the centre of the small circle.

Parameters:
trochoid(t, r1=200, r2=80, offset=20, eh = 'H', pen=['gray'], fill='white',
    filled=False, centred=True, lr='R')

r1 is the (integer) radius of the outer circle.
r2 is the (integer) radius of the inner circle.
offset is the distance of the 'pen' from the centre of the inner circle.
  offset must not be the same as rr (try 0.01 bigger or smaller) for a pattern to draw properly.
eh is either H (hypotrochoid) or E (epitrochoid).
pen is a list of one or more colours which are repeated during drawing.

The number of complete 360 turns the inner circle must make to create a pattern with start and
  finish correctly joined is the numerator of r2/r1 (reduced as far as possible) and the number
  of revolutions it makes within in the outer circle to do so is the denominator.
  Not all combinations of r1, r2, and offset produce a complete pattern.

'''


def main():
    scr = tp.initializescreen(colour='black')
    ada = tp.initializeturtle()
    
    #ada.ht()
    colours = ['red', 'orange', 'yellow', 'light green', 'cyan', 'sky blue', 'violet']
    ada.pensize(2)
    ada.pu()
    ada.setpos(0, 0)
    ada.seth(0)
    ada.pd()
    scr.tracer(100)
    tc.trochoid(ada, 500, 540, 200, 'H', ['green', 'cyan', 'sky blue', 'violet'], 'black', False, True)
    scr.tracer(1)
    ada.pu()
    mb.showinfo('Turtle Graphics Demo', "Click 'OK' for the next trochoid.")
    ada.clear()
    ada.setpos(0, 0)
    ada.seth(0)
    ada.pd()
    ada.pensize(4)
    scr.tracer(2)  
    tc.trochoid(ada, 250, 80, 100, 'H', ['red', 'yellow'], 'midnight blue', True, True, 'L')
    scr.tracer(1)
    ada.pensize(2)
    ada.pu()
    mb.showinfo('Turtle Graphics Demo', "Click 'OK' for the next trochoid.")
    ada.clear()
    ada.setpos(0, 0)
    ada.seth(0)
    ada.pd()
    scr.tracer(1)
    tc.trochoid(ada, 200, 15, 80, 'H', colours, 'black', False, True, 'L')
    scr.tracer(1)
    ada.pu()
    mb.showinfo('Turtle Graphics Demo', "Click 'OK' for the next trochoid.")
    ada.clear()
    ada.setpos(0, 0)
    ada.seth(0)
    ada.pd()
    ada.pensize(4)
    scr.tracer(2)
    tc.trochoid(ada, 260, 120, 80, 'H', colours, 'black', False, True, 'L')
    scr.tracer(1)
    ada.pensize(2)
    ada.pu()
    mb.showinfo('Turtle Graphics Demo', "Click 'OK' for the next trochoid.")
    ada.clear()
    ada.setpos(0, 0)
    ada.seth(18)
    ada.pd()
    ada.pensize(8)
    scr.tracer(2)
    tc.trochoid(ada, 200, 80, 80.01, 'H', ['blue', 'green', 'orange'], 'purple', True, True)
    scr.tracer(1)
    ada.pensize(2)
    ada.pu()
    ada.st()

    scr.exitonclick()


if __name__ == '__main__':
    main()
