'''
Program name :  TurtleUtilsPMP
Author       :  Philip Murray-Pearce
Created      :  19/02/2015
Copyright    :  (c) Philip Murray-Pearce 2015
Licence      :  OSI Simplified BSD
'''


import turtle
import math as m
import tkinter.messagebox as mb


def initializescreen(w=1000, h=850, colour='white', screentitle='Turtle Graphics'):
    s = turtle.Screen()
    s.setup(width=w, height=h, startx=None, starty=None)
    s.title(screentitle)
    s.colormode(255)
    s.bgcolor(colour)
    s.mode('logo')  # Logo mode sets Up as 0°, Right as 90°, Down as 180°, Left as 270°, etc.
    return s


def initializeturtle(shape='turtle', pen='black', fill='white'):
    t = turtle.Turtle()  # Create t as a turtle instance.
    t.shape(shape)       # Draws the turtle as a set shape.
    t.speed(0)           # Sets drawing speed to maximum - others are 1 (slow) - 10 (fast).
    t.clear()            # Clears the screen.
    t.home()             # Puts the turtle at the screen centre pointing Up.
    t.pu()               # Sets pen up (i.e. not drawing).
    t.pencolor(pen)      # Sets the pen colour.
    t.fillcolor(fill)    # Sets the fill colour.
    t.pensize(1)         # Sets the pen width to 1 (minimum).
    return t


def write(t, word, size=14, pen='black', align='center', font='Arial', face='normal', \
    vertc=False, move=False):
    spacing = 1.6
    offset = 0
    if vertc:
        offset = size * spacing
    p = t.isdown()
    if p:
        t.pu()
    lines = word.count('\n') + 1
    t.pencolor(pen)
    t.bk(offset * lines / 2)
    if lines == 1:
        t.write(word, move, align, (font, size, face))
    else:
        t.fd(size * spacing * lines)
        linelist = word.split('\n')
        for i in range(0, lines):
            t.bk(size * spacing)
            t.write(linelist[i], move, align, (font, size, face))
    t.fd(offset * lines / 2)
    if p:
        t.pd()


def gridxy(t, spacing=100, gridcolour='light gray', numbercolour='gray', numbered=True, \
    startx=-500, starty=-500):
    if spacing not in [10, 20, 25, 50, 100]:
        mb.showerror('Grid parameters error!', 'Grid spacing must be 10, 20, 25, 50, or 100.')
        return
    oldx, oldy = t.pos()
    oldheading = t.heading()
    oldpencolor = t.pencolor()
    oldpensize = t.pensize()
    t.pencolor(gridcolour)
    t.pensize(1)
    hv = [startx, starty]
    for k in range(2):
        t.pu()
        t.setpos(startx, starty)
        t.pd()
        t.seth(90 * k)
        for i in range(0, abs(hv[(k+1)%2])*2, spacing):
            t.pensize(1)
            if i%100 == 0 and spacing < 100:
                t.pensize(2)
            t.lt(90 * (k*2-1))
            t.fd(abs(hv[k])*2)
            if i == abs(hv[(k+1)%2]) and numbered:
                t.bk(abs(hv[k])*2)
                t.pencolor(numbercolour)
                spacingn = spacing
                if spacing in [10, 20]:
                    spacingn = 100
                for j in range(hv[k], abs(hv[k]), spacingn):
                    t.write(str(j))
                    t.fd(spacingn)
                t.pencolor(gridcolour)
            t.bk(abs(hv[k])*2)
            t.rt(90 * (k*2-1))
            t.fd(spacing)
        t.pu()
    t.pencolor(oldpencolor)
    t.pensize(oldpensize)
    t.seth(oldheading)
    t.setpos(oldx, oldy)


def polygon(t, sides=4, size=100, pen='black', fill='white', filled=False, centred=False, lr='R'):
    if lr not in ['L','R']:
        mb.showerror('Polygon parameters error!', 'Direction must be R or L.')
        return
    if size < 1:
        mb.showerror('Polygon parameters error!', \
            'The length of each side must not be less than 1.')
        return
    if sides < 3:
        mb.showerror('Polygon parameters error!', \
            'The number of sides must not be less than 3.')
        return
    if centred:
        lr = 'R'
        r = size/(2*m.sin(m.radians(180/sides)))
        t.pu()
        t.fd(r)
        t.rt(90+180/sides)
        t.pd()
    t.pencolor(pen)
    t.pd()
    if filled:
        t.fillcolor(fill)
        t.begin_fill()
    for i in range(sides):
        t.fd(size)
        if lr == 'L':
            t.lt(360/sides)
        else:
            t.rt(360/sides)
    if filled:
        t.end_fill()
    if centred:
        t.pu()
        t.rt(90-180/sides)
        t.fd(r)
        t.rt(180)
    t.pu()


def square(t, size=100, pen='black', fill='white', filled=False, centred=False, lr='R'):
    if filled:
        t.fillcolor(fill)
        t.begin_fill()
    polygon(t, 4, size, pen, fill, filled, centred, lr)
    if filled:
        t.end_fill()


def rectangle(t, side1=150, side2=100, pen='black', fill='white', filled=False, centred=False, \
    lr='R', oval=False):
    if lr not in ['L','R']:
        mb.showerror('Rectangle parameters error!', 'Direction must be R or L.')
        return
    if side1 < 1 or side2 < 1:
        mb.showerror('Rectangle parameters error!', \
            'The length of each side must not be less than 1.')
        return
    if centred:
        lr = 'R'
        t.pu()
        t.bk(side1 / 2)
        t.lt(90)
        t.fd(side2 / 2)
        t.rt(90)
        t.pd()
    x = 1
    if lr == 'L':
        x = -1
    t.pd()
    t.pencolor(pen)
    if filled:
        t.fillcolor(fill)
        t.begin_fill()
    for i in range(2):
        if oval:
            t.fd(side1)
            t.circle(side2/2*-x, 180)
        else:
            t.fd(side1)
            t.rt(90*x)
            t.fd(side2)
            t.rt(90*x)
    if filled:
        t.end_fill()
    t.pu()
    if centred:
        t.fd(side1 / 2)
        t.rt(90)
        t.fd(side2 / 2)
        t.lt(90)


def oval(t, side1=150, side2=100, pen='black', fill='white', filled=False, centred=False, lr='R'):
    rectangle(t, side1, side2, pen, fill, filled, centred, lr, True)


def segment(t, radius=100, angle=90, pen='black', fill='white', filled=False, lr='R'):
    if lr not in ['L','R']:
        mb.showerror('Segment parameters error!', 'Direction must be R or L.')
        return
    if angle < 0 or angle > 360:
        mb.showerror('Segment parameters error!', \
            'The angle must not be less than 0 or more than 360.')
        return
    if radius < 1:
        mb.showerror('Segment parameters error!', 'The radius must not be less than 1.')
        return
    if lr == 'R':
        radius = -radius
    t.pencolor(pen)
    t.pd()
    if filled:
        t.fillcolor(fill)
        t.begin_fill()
    t.circle(radius, angle)
    if filled:
        t.end_fill()
    t.pu()


def sector(t, radius=100, angle=90, pen='black', fill='white', filled=False, centred=True, lr='R'):
    if lr not in ['L','R']:
        mb.showerror('Sector parameters error!', 'Direction must be R or L.')
        return
    if angle < 0 or angle > 360:
        mb.showerror('Sector parameters error!', \
            'The angle must not be less than 0 or more than 360.')
        return
    if radius < 1:
        mb.showerror('Sector parameters error!', 'The radius must not be less than 1.')
        return
    angle = angle % 360
    if angle == 0:
        angle = 360
    if angle != 360:
        centred = True
    x = 1
    if lr == 'L':
        x = -1
    t.pencolor(pen)
    t.pd()
    if filled:
        t.fillcolor(fill)
        t.begin_fill()
    if angle == 360:
        t.pu()
    if centred:
        t.fd(radius)
        t.rt(90*x)
    t.pd()
    t.circle(-radius*x, angle)
    if angle == 360:
        t.pu()
    if centred:
        t.rt(90*x)
        t.fd(radius)
        t.rt((180-angle)*x)
    if filled:
        t.end_fill()
    t.pu()


def circle(t, radius=100, pen='black', fill='white', filled=False, centred=False, lr='R'):
    sector(t, radius, 360, pen, fill, filled, centred, lr)


def ellipse(t, a=100, b=50, s=20, pen='black', fill='white', filled=False, centred=False, \
    lr='R', semi=False):
    if lr not in ['L','R']:
        mb.showerror('Ellipse parameters error!', 'Direction must be R or L.')
        return
    if a < b:
        mb.showerror('Ellipse parameters error!', \
            'The length of the first axis must be greater than the length of the second axis.')
        return
    if s < 4:
        mb.showerror('Ellipse parameters error!', "The number of 'sides' must be 4 or more.")
        return
    if s/2 != s//2:
        mb.showerror('Ellipse parameters error!', "The number of 'sides' must be even.")
        return
    if a < 1 or b < 1:
        mb.showerror('Ellipse parameters error!', \
            'The length of each axis must not be less than 1.')
        return
    x = 1
    if lr == 'L':
        x = -1
    if centred:
        t.pu()
        t.lt(90*x)
        t.fd(a)
        t.rt(90*x)
    t.pencolor(pen)
    t.pd()
    if filled:
        t.fillcolor(fill)
        t.begin_fill()
    stp = 360/s
    for k in range(2-semi):
        c0 = a
        c1 = a * m.cos(m.radians(stp))
        s0 = 0
        s1 = b * m.sin(m.radians(stp))
        sd1 = m.sqrt((c1-c0)*(c1-c0) + (s1-s0)*(s1-s0))
        t.rt(m.degrees(m.acos(s1/sd1))*x)
        for i in range(int(s/2)):
            c2 = a * m.cos(m.radians((i+2)*stp))
            s2 = b * m.sin(m.radians((i+2)*stp))
            sd2 = m.sqrt((c2-c1)*(c2-c1) + (s2-s1)*(s2-s1))
            chord = m.sqrt((c2-c0)*(c2-c0) + (s2-s0)*(s2-s0))
            turnangle = 180 - m.degrees(m.acos((sd1*sd1 + sd2*sd2 - chord*chord)/(2*sd1*sd2)))
            t.fd(sd1)
            t.rt(turnangle*x)
            c0 = c1
            c1 = c2
            s0 = s1
            s1 = s2
            sd1 = sd2
        t.lt(turnangle/2*x)
    if filled:
        t.end_fill()
    t.pu()
    if centred:
        t.rt(90*x)
        t.fd(a)
        t.lt(90*x)


def spiral(t, sides=4, size=10, gap=2, pen='black', lr='R'):
    if lr not in ['L','R']:
        mb.showerror('Spiral parameters error!', 'Direction must be R or L.')
        return
    if sides < 3 or sides > 20:
        mb.showerror('Spiral parameters error!', \
            "The number of 'sides' must not be less than 3 or more than 20.")
        return
    if gap < 1:
        mb.showerror('Spiral parameters error!', 'The gap between turns must not be less than 1.')
        return
    if size < 1:
        mb.showerror('Spiral parameters error!', 'The size must not be less than 1.')
        return
    x = gap
    d = 1
    if lr == 'L':
        d = -1
    t.pencolor(pen)
    t.pd()
    for i in range(size*sides):
        t.fd(x)
        t.rt(d*360/sides)
        x = x+gap*(2/(sides-2))**1.5
    t.pu()


def vesica(t, size=100, angle=90, pen='black', fill='white', filled=False):
    t.lt(angle/2)
    if size < 10:
        mb.showerror('Vesica parameters error!', 'The size must not be less than 10.')
        return
    if angle < 30 or angle > 150:
        mb.showerror('Vesica parameters error!', \
            'The angle must not be less than 30 or more than 150.')
        return
    t.pd()
    segment(t, size, angle, pen, fill, filled)
    t.rt(180-angle)
    segment(t, size, angle, pen, fill, filled)
    t.rt(180-angle/2)
    t.pu()


def star(t, points=5, size=100, pen='black', fill='white', filled=False, lr='R'):
    if lr not in ['L','R']:
        mb.showerror('Star parameters error!', 'Direction must be R or L.')
        return
    if points < 4:
        mb.showerror('Star parameters error!', 'A star must have 4 or more points.')
        return
    if size < 1:
        mb.showerror('Star parameters error!', 'The size must not be less than 1.')
        return
    x = 1
    if lr == 'L':
        x = -1
    p = 1
    if points % 2 == 0:
        p = 2
    t.pencolor(pen)
    t.pd()
    t.rt(90/points*x)
    if filled == True:
        t.fillcolor(fill)
        t.begin_fill()
    for i in range(points*p):
        t.fd(size)
        t.rt(180-180/points*x)
    if filled == True:
        t.end_fill()
    t.pu()


def crescent(t, size=100, angle=90, pen='black', fill='white', filled=False, lr='R'):
    if lr not in ['L','R']:
        mb.showerror('Crescent parameters error!', 'Direction must be R or L.')
        return
    if size < 10:
        mb.showerror('Crescent parameters error!', 'The size not be less than 10.')
        return
    if angle < 30 or angle > 120:
        mb.showerror('Crescent parameters error!', \
            'The angle must not be less than 30 or more than 120.')
        return
    x = 1
    if lr == 'L':
        x = -1
    t.pencolor(pen)
    t.pd()
    if filled:
        t.fillcolor(fill)
        t.begin_fill()
    t.rt(angle / 2 * x)
    t.circle(size * x, angle)
    t.rt((180 - angle / 2) * x)
    r = size / (2 * m.cos(m.radians(angle / 2)))
    t.circle(-r * x, angle * 2)
    t.rt((180 - angle) * x)
    if filled:
        t.end_fill()
    t.pu()


'''

Main Python Turtle Commands

t.clear()       Clear screen
t.home()        Put turtle at screen centre pointing up
t.speed()       Set speed of turtle movement - has one parameter (0-10)
t.st()          Show turtle
t.ht()          Hide turtle (drawing is faster with turtle hidden)
t.setpos()      Set turtle position - has 2 parameters for x and y position
t.seth()        Set turtle heading - has 1 parameter (0-360)
t.fd()          Move forwards - has 1 parameter for distance
t.bk()          Move backward - has 1 parameter for distance
t.rt()          Turn right - has 1 parameter for number of degrees
t.lt()          Turn left  - has 1 parameter for number of degrees
t.circle()      Draw an arc - has 2 main parameters for radius and angle turned
t.pd()          Pen down
t.pu()          Pen up
t.pensize()     Pen width   - has 1 parameter for width
t.pencolor()    Pen colour  - has 1 constant or 3 RGB values as parameters
t.fillcolor()   Fill colour - has 1 constant or 3 RGB values as parameters
t.begin_fill()  Shapes drawn between this and t.end_fill are filled in
t.end_fill()    Ends filling in started by t.begin_fill
t.dot()         Put a dot at the current position
t.stamp()       Put a turtle at the current position and heading
t.write()       Writes text - details of text, alignment, font, etc. as parameters


PMP Turtle Utils Commands

initializescreen(width=1000, height=850, colour='white', title='Turtle Graphics')
initializeturtle(shape='turtle', pen='black', fill='white')
write(turtle, text, fontsize=14, pen='black', align='center', font='Arial', face='normal')
gridxy(t, spacing=100, gridcolour='light gray', numbercolour='gray', numbered=True,
    startx=-500, starty=-500)
polygon(turtle, sides, size, pen='black', fill='white', filled=False, centred=False, lr='R')
square(turtle, size, pen='black', fill='white', filled=False, centred=False, lr='R')
rectangle(turtle, width, height, pen='black', fill='white', filled=False, centred=False,
    lr='R', oval=False)
oval(turtle, width, height, pen='black', fill='white', filled=False, centred=False, lr='R')
segment(turtle, radius, angle, pen='black', fill='white', filled=False, lr='R')
sector(turtle, radius, angle, pen='black', fill='white', filled=False, centred=True, lr='R')
circle(turtle, radius, pen='black', fill='white', filled=False, centred=True, lr='R')
ellipse(turtle, long radius, short radius, sides=20, pen='black', fill='white',
    filled=False, centred=False, lr='R', semi=False)
vesica(turtle, size [>=10], angle [30-159], pen='black', fill='white', filled=False) 
spiral(turtle, sides=4, size=100, gap=1, pen='black', lr='R')
star(turtle, points, size, pen='black', fill='white', filled=False, lr='R')
crescent(turtle, size [>=10], angle [30-120], pen='black', fill='white', filled=False, lr='R')

'''
